; The XML-in-LAML manual style and SchemeDoc.
; This file depends on styles/manual/manual-kernel.scm, which is the original
; LAML manual facility brought up to date by using the XHTML mirror library instead of the
; old ad hoc HTML LAML libraries.
; 
; Processes an XML-in-LAML manual file and generates an XHTML manual with
; CSS styling.

; ---------------------------------------------------------------------------------------------------
; Constants

(define old-style-manual-directory (string-append laml-dir "styles/manual/"))
(define manual-software-directory (string-append laml-dir "styles/manual/"))
(define manual-source-directory (startup-directory))

(define manual-scheme-documentation-commenting-style #f) ; assigned later

; ---------------------------------------------------------------------------------------------------
; Loading

; if conservative-xhtml-loading do not unnecessarily reload xhtml10-transitional.
(if (eq? laml-load-variation 'conservative-xhtml-loading)
    (begin
      (if (not (memq 'xhtml10-transitional (languages-in-use)))
          (lib-load "xml-in-laml/mirrors/xhtml10-transitional-mirror.scm"))

    )   
    (begin
      (lib-load "xml-in-laml/xml-in-laml.scm")
      (lib-load "xml-in-laml/mirrors/xhtml10-transitional-mirror.scm")
    )
)


(lib-load "xhtml1.0-convenience.scm")
(load (string-append laml-dir "styles/xml-in-laml/manual/mirror/manual-1-mirror.scm"))

(lib-load "color.scm")
(lib-load "time.scm")
(lib-load "file-read.scm")

(load (string-append laml-dir "tools/schemedoc" "/" "schemedoc.scm"))
(set! delete-comment-file? #t)

(load (string-append old-style-manual-directory "manual-kernel.scm"))
(load (string-append laml-dir "tools/dtd-parser/element-attribute-access.scm"))


; ---------------------------------------------------------------------------------------------------

; Action procedure:

(define (manual! manual-ast)
  (manual-briding! manual-ast)
)

; ---------------------------------------------------------------------------------------------------

(define (type-of-manual manual-ast)
  (let ((manual-page-list (find-asts manual-ast "manual-page"))
        (manual-section-list (find-asts manual-ast "manual-section")))
    (cond ((and (find-first-ast manual-ast "manual-from-scheme-file")     'manual-from-scheme-file))
          ((and (find-first-ast manual-ast "manual-from-xml-dtd")
                (null? manual-page-list) (null? manual-section-list))     'manual-from-xml-dtd)
          ((find-first-ast manual-ast "merged-manual-from-xml-dtd")              'merged-manual-from-xml-dtd)
          (else                                                           'manual-pages-sections-only))))

(define (manual-briding! manual-ast)
  (let* ((man-title (find-first-ast manual-ast "manual-title" ast-subtrees))
         (manual-aut (find-first-ast manual-ast "manual-author" ast-subtrees))
         (manual-affil (find-first-ast manual-ast "manual-affiliation" ast-subtrees))
         (manual-abstr (find-first-ast manual-ast "manual-abstract" ast-subtrees))
         (laml-resource (as-boolean (unique-ast-attribute manual-ast 'laml-resource "false")))

         (manual-type (type-of-manual manual-ast))
         (attribute-explanation-default (unique-ast-attribute manual-ast 'default-attribute-explanation "-"))
         (documentation-commenting-style-attr
            (as-symbol (unique-ast-attribute manual-ast 'documentation-commenting-style "multi-semicolon")))
         (keep-syntactical-comment-file-attr (as-boolean (unique-ast-attribute manual-ast 'keep-syntactical-comment-file "false")))
         (css-stylesheet (unique-ast-attribute manual-ast 'css-stylesheet #f))
         (css-prestylesheet (unique-ast-attribute manual-ast 'css-prestylesheet #f))
         (css-copying? 
           (as-boolean 
            (unique-ast-attribute manual-ast 'css-stylesheet-copying
             (if laml-resource #f #t))))
         (manual-destination-name-attr (unique-ast-attribute manual-ast 'manual-destination-name #f))
         (attribute-sorting-attr (as-symbol (unique-ast-attribute manual-ast 'attribute-sorting "as-in-dtd")))
         (scheme-source-linking-attr (as-boolean (unique-ast-attribute manual-ast 'scheme-source-linking "false")))
        )

    ; Handle overall manual manual settings:
    (if man-title (set! the-manual-title man-title))

    ; Assignment of schemedoc extracting variable: defined in tools/schemedoc/schemedoc.scm
    (set! scheme-documentation-commenting-style documentation-commenting-style-attr)
    (set! manual-scheme-documentation-commenting-style documentation-commenting-style-attr) ; also set a manual variable

    ; Assignment of schemedoc extracting variable: defined in tools/schemedoc/schemedoc.scm
    (set! delete-comment-file? #f)   ; (not keep-syntactical-comment-file-attr)  ; !!!!
    
    (cond ((and manual-aut manual-affil)
             (set! the-manual-author (append manual-aut manual-affil)))
          (manual-aut
             (set! the-manual-author manual-aut))
          (manual-affil
             (set! the-manual-author manual-affil))
          (else (set! the-manual-author '())))

    (if manual-abstr (set! the-manual-abstract manual-abstr))

    ; some of the assignments below may be redone in manual-briding-scheme-source1
    ; based on values extracted from the documetation abstract in the Scheme source file.
    (set! laml-manual-stuff laml-resource)
    (set! standard-attribute-explanation attribute-explanation-default)
    (if css-stylesheet (set! the-manual-stylesheet css-stylesheet))
    (if css-prestylesheet (set! the-manual-prestylesheet css-prestylesheet))
    (set! copy-css-stylesheet? css-copying?)
    (set! css-stylesheet-schema (if laml-manual-stuff 'laml 'local))
    (if manual-destination-name-attr (set! manual-name manual-destination-name-attr)) ; instead of laml source file name
    (set! attribute-sorting-kind attribute-sorting-attr)
    (set! link-to-ep-source-program? scheme-source-linking-attr) 

    ; Handle all manual-type specific data:
    (cond ((eq? manual-type 'manual-pages-sections-only)
             (manual-briding-standard! manual-ast))
          ((eq? manual-type 'manual-from-scheme-file)
             (manual-briding-scheme-source! manual-ast))
          ((eq? manual-type 'manual-from-xml-dtd)
             (manual-briding-xml-dtd! manual-ast))
          ((eq? manual-type 'merged-manual-from-xml-dtd)
             (manual-briding-merged-xml-dtd! manual-ast))
          (else (laml-error "manual-briding!: Unknown kind of manual type" manual-type)))

  )
)

; Manual sections and manual pages only.
(define (manual-briding-standard! manual-ast)
  (let ((manual-destination-path (get-manual-destination-path manual-ast))
        (doc-list
           (traverse-and-collect-all-from-ast 
              manual-ast
              (lambda (ast) (or (equal? "manual-page" (ast-element-name ast))
                                (equal? "manual-section" (ast-element-name ast))))
              transform-page-or-section)))
    (set! the-manual-destination-path manual-destination-path)
    (make-manual (reverse doc-list) 'manual-pages-sections-only  manual-destination-path )  

   )              
)

; Return the absolute destination path, taken from one of the front matters attributes,
; source-destination-delta or manual-destination-directory. 
(define (get-manual-destination-path manual-ast)
 (let ((abs-path (unique-ast-attribute manual-ast "manual-destination-directory" #f))
       (rel-path (unique-ast-attribute manual-ast "source-destination-delta" #f)))
  (cond (rel-path (string-append (startup-directory) rel-path))
        (abs-path abs-path)
        (else (startup-directory)))))

(define (transform-page-or-section page-or-section-ast)
 (cond ((equal? "manual-page" (ast-element-name page-or-section-ast)) 
            (transform-manual-page page-or-section-ast))
       ((equal? "manual-section" (ast-element-name page-or-section-ast)) 
            (transform-manual-section  page-or-section-ast))
       (else (laml-error "transform-page-or-section: Expects AST of type manual-page or manual-section. Got:" 
                         (ast-element-name page-or-section-ast)))))

(define (transform-manual-page page-ast)
  (let* ((name-attr (ast-attribute page-ast 'name))
         (page-subclause-list (filter ast? (ast-subtrees page-ast)))
         (external-reference-clauses (filter (ast-of-type? 'element-name "external-reference") page-subclause-list))
         (internal-references-clauses (filter (ast-of-type? 'element-name "internal-references") page-subclause-list))
         (other-clauses 
           (filter 
             (lambda (ast) (and (not (equal? (ast-element-name ast) "external-reference")) 
                           (not (equal? (ast-element-name ast) "internal-references"))))
             page-subclause-list))

         (transformed-clauses
           (append 
              (map clause-transform other-clauses) 
              (list 
                (cons 'cross-references 
                      (append (map clause-transform external-reference-clauses)
                              (map clause-transform internal-references-clauses))))))
        )
    (cons (list 'kind "manual-page") (cons (list 'title name-attr) transformed-clauses))
  )
)


(define (transform-manual-section section-ast)
  (let ((section-id (ast-attribute section-ast 'id #f))
        (section-subclause-list (filter ast? (ast-subtrees section-ast))))
    (if section-id
        (cons (list 'section-id section-id)
              (cons (list 'kind "manual-section")
                    (map clause-transform section-subclause-list)))
        (cons (list 'kind "manual-section")
          (map clause-transform section-subclause-list)))))

(define (clause-transform page-clause)
 (let ((transformer (find-transformer (as-symbol (ast-element-name page-clause)))))
    (transformer page-clause)))

(define (find-transformer name)
  (cond 
        ((eq? 'section-title name) section-title-transformer)
        ((eq? 'section-body name) section-body-transformer)
        ((eq? 'title name) title-transformer)
        ((eq? 'form name) form-transformer)
        ((eq? 'description name) description-transformer)
        ((eq? 'example-form name) example-form-transformer)
        ((eq? 'precondition name) precondition-transformer)
        ((eq? 'postcondition name) postcondition-transformer)
        ((eq? 'parameters name) parameters-transformer)
        ((eq? 'example name) example-transformer)
        ((eq? 'external-reference name) external-reference-transformer)
        ((eq? 'internal-references name) internal-references-transformer)
        ((eq? 'comment name) comment-transformer)
        ((eq? 'misc name) misc-transformer)
        ((eq? 'returns name) returns-transformer)
        ((eq? 'attributes name) attributes-transformer)
        ((eq? 'attribute-descriptions name) attribute-descriptions-transformer)
        (else (laml-error "find-transformer: Cannot locate transformation function for element of name" name))))

(define (section-title-transformer ast)
  (list 'section-title (aggregated-contents ast)))  ; abstraher igen

(define (section-body-transformer ast)
  (list 'section-body (aggregated-contents ast)))

(define (title-transformer ast)
  (list 'title (aggregated-contents ast)))

(define (form-transformer ast)
  (list 'form (aggregated-contents ast)))

(define (description-transformer ast)
  (list 'description (aggregated-contents ast)))

(define (example-form-transformer ast)
  (list 'xml-in-laml-example-form (aggregated-contents ast)))

(define (precondition-transformer ast)
  (list 'pre-condition (aggregated-contents ast)))

(define (postcondition-transformer ast)
  (list 'post-condition (aggregated-contents ast)))

(define (parameters-transformer parameters-ast)
  (cons 'parameters
        (traverse-and-collect-all-from-ast 
             parameters-ast
             (ast-of-type? 'element-name "parameter")
             (lambda (par-ast) (list 'parameter (ast-attribute par-ast 'name) (aggregated-contents par-ast))))))

(define (example-transformer ast)
 (list 'example (aggregated-contents ast)))

(define (external-reference-transformer external-reference-ast)
  (list 'reference
        (ast-attribute external-reference-ast 'category)
        (aggregated-contents external-reference-ast)
        (ast-attribute external-reference-ast 'href)))

(define (internal-references-transformer internal-references-ast)
  (append 
     (list 'internal-references (ast-attribute internal-references-ast 'category))
     (traverse-and-collect-all-from-ast 
        internal-references-ast 
        (ast-of-type? 'element-name "name-ref")
        (lambda (name-ref-ast) (aggregated-contents name-ref-ast)))))

(define (comment-transformer ast)
 (list 'comment (aggregated-contents ast)))

(define (misc-transformer ast)
 (list 'misc (aggregated-contents ast)))

(define (returns-transformer ast)
 (list 'returns (aggregated-contents ast)))

(define (attributes-transformer attributes-ast)
  (cons 'attributes
        (traverse-and-collect-all-from-ast 
             attributes-ast
             (ast-of-type? 'element-name "attribute")
             (lambda (attr-ast) (list 'attribute (ast-attribute attr-ast 'name) 
                                      (ast-attribute attr-ast 'type) (ast-attribute attr-ast 'default-status)
                                      (aggregated-contents attr-ast))))))

(define (attribute-descriptions-transformer attribute-descriptions-ast)
  (cons 'attribute-descriptions
        (traverse-and-collect-all-from-ast 
             attribute-descriptions-ast
             (ast-of-type? 'element-name "attribute-description")
             (lambda (attr-ast) (list 'attribute-description (ast-attribute attr-ast 'name) 
                                      (aggregated-contents attr-ast))))))

; Manual from scheme source only.
(define (manual-briding-scheme-source! manual-ast)
 (let* ((laml-resource (unique-ast-attribute manual-ast 'laml-resource #f))
        (keep-syntactical-comment-file-attr (unique-ast-attribute manual-ast 'keep-syntactical-comment-file #f))
        (css-stylesheet (unique-ast-attribute manual-ast 'css-stylesheet #f))
        (css-prestylesheet (unique-ast-attribute manual-ast 'css-prestylesheet #f))
        (css-copying? (unique-ast-attribute manual-ast 'css-stylesheet-copying #f))
        (abs-path (unique-ast-attribute manual-ast "manual-destination-directory" #f))
        (rel-path (unique-ast-attribute manual-ast "source-destination-delta" #f))
        (source-linking? (unique-ast-attribute manual-ast "scheme-source-linking" #f))  
       )
  (let* ((is-empty? (lambda (x) (or (and (string? x) (empty-string? x)) (and (boolean? x) (not x)) (and (list? x) (null? x)))))  

         (manual-from-scheme-file-clause (find-first-ast manual-ast "manual-from-scheme-file"))
         (manual-source-path (ast-attribute manual-from-scheme-file-clause 'src))
         (final-manual-source-path (if (absolute-file-path? manual-source-path)
                                       manual-source-path
                                       (string-append (startup-directory) manual-source-path)))
         (doc-list (extract-documentation-from-scheme-file final-manual-source-path))
         (doc-list-1 
            (if (or link-to-ep-source-program?
                    (and (not source-linking?)   ; no scheme-source-linking attribute in LAML file
                         (as-boolean extracted-scheme-source-linking) ; asked for scheme-source-linking in scheme source file
                    ))
                (let* ((ep-dir-name (string-append manual-name "-" "ep"))
                       (version-number 1)
                       (ep-html-file-name (string-append manual-name "-" (as-string version-number)))
                      )
                   (extend-doc-list-with-source-references doc-list ep-dir-name ep-html-file-name))
                doc-list))
         (manual-destination-path (if (or abs-path rel-path) 
                                      (get-manual-destination-path manual-ast)
                                      (string-append (startup-directory) extracted-source-destination-delta))) 

        )
     (set! the-manual-destination-path manual-destination-path)

     ; Use extracted manual information, assigned by extract-documentation-from-scheme-file.
     ; Only use this information if it is not explicitly supplied in the front matters.
     (if (is-empty? the-manual-abstract) (set! the-manual-abstract extracted-manual-abstract)) 
     (if (is-empty? the-manual-title) (set! the-manual-title extracted-manual-title))
     (if (is-empty? the-manual-author) (set! the-manual-author (list extracted-manual-author extracted-manual-affiliation)))

     (if (not laml-resource) (set! laml-manual-stuff (as-boolean extracted-laml-resource)))
     (if (not css-prestylesheet) (set! the-manual-prestylesheet extracted-css-prestylesheet))
     (if (not css-stylesheet) (set! the-manual-stylesheet extracted-css-stylesheet))
     (if (not css-copying?) (set! copy-css-stylesheet? (as-boolean extracted-css-stylesheet-copying)))

     (make-manual doc-list-1 'manual-from-scheme-file manual-destination-path (adapt-informative-path final-manual-source-path))

     (if (or link-to-ep-source-program?
	     (and (not source-linking?) ; no scheme-source-linking attribute in LAML file
		  (as-boolean extracted-scheme-source-linking) ; asked for scheme-source-linking in scheme source file
		  ))
         (make-ep-source-program manual-name final-manual-source-path manual-destination-path))  
  )
 )
)

; Manual from XML DTD only.
(define (manual-briding-xml-dtd! manual-ast)
  (let* ((manual-from-xml-dtd-clause (find-first-ast manual-ast "manual-from-xml-dtd"))
         (dtd-source-path (ast-attribute manual-from-xml-dtd-clause 'src))
         (dtd-source-path-1 (string-append (file-name-initial-path dtd-source-path) (file-name-proper dtd-source-path)))
         (mirror-names-defined-attr (as-boolean (unique-ast-attribute manual-ast "mirror-names-defined" "true")))
         (mirror-name-prefix-attr (unique-ast-attribute manual-ast "mirror-name-prefix" ""))
         (language-name (file-name-proper dtd-source-path))
         (dtd-list (file-read (string-append (startup-directory) dtd-source-path-1 "." "lsp")))
         (doc-list
           (let ((entries (map 
                          (manual-extend 'description (string-append "An XML element as defined in the " language-name " XML DTD."))
                          (manual-from-parsed-dtd dtd-list mirror-name-prefix-attr))))
            (if mirror-names-defined-attr
                entries
                (map 
                 (manual-extend 'misc "This mirror function is solely available via the language map.")
                 entries))
               ))
         (manual-destination-path (get-manual-destination-path manual-ast))
         (make-manual-template? (as-boolean (unique-ast-attribute manual-ast "make-dtd-manual-template" #f)))
         (template-file-name (ensure-non-existing-file-in-dir  "manual-template.laml" manual-destination-path))

       )

    (set! the-manual-destination-path manual-destination-path)
     
    ; Generate manual template on new fresh laml file:
    (if make-manual-template?
     (write-text-file
       (manual-template-from-parsed-dtd dtd-list)
       (string-append manual-destination-path template-file-name)))
    
    (make-manual (reverse doc-list) 'manual-from-xml-dtd manual-destination-path))
)

; Manual controlled from manual pages and manual sections, merged with manual contributions from an XML dtd
(define (manual-briding-merged-xml-dtd! manual-ast)
  (let* ((manual-from-xml-dtd-clause (find-first-ast manual-ast "merged-manual-from-xml-dtd"))
         (dtd-source-path (ast-attribute manual-from-xml-dtd-clause 'src))
         (dtd-source-path-1 (string-append (file-name-initial-path dtd-source-path) (file-name-proper dtd-source-path)))
         (language-name (file-name-proper dtd-source-path))
         (page-section-doc-list
           (traverse-and-collect-all-from-ast 
              manual-ast
              (lambda (ast) (or (equal? "manual-page" (ast-element-name ast))
                                (equal? "manual-section" (ast-element-name ast))))
              transform-page-or-section)) 
         (dtd-list (file-read (string-append (startup-directory) dtd-source-path-1 "." "lsp")))
         (dtd-doc-list (manual-from-parsed-dtd dtd-list ""))
         (manual-destination-path (get-manual-destination-path manual-ast))
         (reversed-page-section-list (reverse page-section-doc-list))
         (merged-manual-pages (merge-manual-pages reversed-page-section-list dtd-doc-list))
         (make-manual-template? (as-boolean (unique-ast-attribute manual-ast "make-dtd-manual-template" #f)))
         (template-file-name (ensure-non-existing-file-in-dir  "manual-template.laml" manual-destination-path))
        )

    (set! the-manual-destination-path manual-destination-path)

    (merged-xml-dtd-completeness-control! reversed-page-section-list dtd-doc-list)

    ; Generate manual template on new fresh laml file:
    (if make-manual-template?
     (write-text-file
       (manual-template-from-parsed-dtd dtd-list)
       (string-append manual-destination-path template-file-name)))

    (make-manual merged-manual-pages 'merged-manual-from-xml-dtd manual-destination-path))
)

; Aggregave xml-crossing to a single string or a single ast.
(define (aggregate-xml-crossing xml-crossing)
  (cond ((ast? xml-crossing) xml-crossing)
        ((string? xml-crossing) xml-crossing) 
        ((and (list? xml-crossing) (= 1 (length xml-crossing)) (ast? (car xml-crossing))) (car xml-crossing))
        ((and (list? xml-crossing) (= 1 (length xml-crossing)) (string? (car xml-crossing))) (car xml-crossing))
        ((list? xml-crossing) (span xml-crossing))
        (else "aggregate-xml-crossing: Unknown kind of input:" xml-crossing)))

(define (aggregated-contents ast)
  (aggregate-xml-crossing (ast-subtrees ast)))

; ---------------------------------------------------------------------------------------------------
; Manual template generation - for DTD manual merging.


; Return a manual template from the xml dtd-list-structure, in which
; attribute descriptions etc. can be inserted. Returns a text string.
(define (manual-template-from-parsed-dtd dtd-list-structure)
 (let* ((attributes (filter (lambda (x) (eq? 'attribute (car x))) dtd-list-structure))
        (elements   (filter (lambda (x) (eq? 'element (car x))) dtd-list-structure))
        (element-names (map cadr elements))
        (sorted-attributes (sort-list attributes (generate-leq element-names cadr string=?)))
       )
  (string-append
   "(load (string-append laml-dir \"laml.scm\"))" CR
   "(laml-style \"xml-in-laml/manual/manual\")" CR
   "(manual"  CR
   "  (manual-front-matters"  CR
   "    (manual-title \"\")"  CR
   "    (manual-author \"\")"  CR
   "    (manual-affiliation \"\")"  CR
   "    (manual-abstract \"\")"  CR
   "    'css-prestylesheet \"normal\""  CR
   "    'css-stylesheet \"original\""  CR
   "  )" CR CR
   
    (list-to-string
     (map manual-template-page-from-attribute sorted-attributes)
     (string-append CR)) CR

   "  (merged-manual-from-xml-dtd 'src \"\")" CR CR

   ")"  CR)))

(define (manual-template-page-from-attribute dtd-attribute-structure)
  (let ((element-name (attribute-name-dtd dtd-attribute-structure))
        (attr-list (attribute-list-dtd dtd-attribute-structure)))
    (string-append
   "  (manual-page 'name " (string-it (as-string element-name))  CR  
   "    (description \"\")"  CR  
   "    (attribute-descriptions "  CR   
             
    (list-to-string
     (map attribute-description-template (map first attr-list))
     (string-append CR)) CR

   "    )" CR
   "  )" CR)))

(define (attribute-description-template attr-name)
  (string-append
    "      (attribute-description 'name " (string-it (as-string attr-name)) " \"\")"))
       
; ---------------------------------------------------------------------------------------------------

(define (adapt-informative-path path)
  (if laml-manual-stuff
      (if (looking-at-substring? path 0 laml-dir)
          (normalize-relative-file-path (substring path (string-length laml-dir) (string-length path)))
          path)
      path))



; ---------------------------------------------------------------------------------------------------------------
; EP Interfacing


(define (in-laml-temp suffix)
  (string-append laml-dir "temp/" suffix))

(define fixed-image-directory #f) ; assigned later

; manual-name is the proper name of the manual (typically source-file-name-without-extension).
; original-source-file-path is the full path to the original source file (with original lexical comments).
; manual-destination-path is the full file path to the directory in which the HTML manual is generated.
(define (make-ep-source-program manual-name original-source-file-path manual-destination-path)
  (load (string-append laml-dir "styles/xml-in-laml/elucidator-2/elucidator.scm"))  ; Perhaps move

  (let* ((source-file-path-syntactical-comments (in-laml-temp "temp-source-file.scm"))
         (source-key manual-name) 
         (local-ep-dir-name (string-append manual-name "-ep"))
         (ep-dir-path (string-append manual-destination-path local-ep-dir-name "/"))
         (version-number 1)
         (html-destination-file-path (string-append ep-dir-path  manual-name  "-" (as-string version-number) "." "html"))         
        )

   ; Create EP directory:
   (ensure-directory-existence! manual-destination-path local-ep-dir-name)

   ; (Re)make syntactical comments:
   (set! scheme-documentation-commenting-style manual-scheme-documentation-commenting-style)
   (lexical-to-syntactical-comments! original-source-file-path source-file-path-syntactical-comments)

   (let ((source-list (read-source source-file-path-syntactical-comments source-key version-number)))

     ; Establish necessary EP context
     (if laml-manual-stuff
         (begin   ; access images from central location
           (set! source-directory (string-append manual-destination-path local-ep-dir-name "/"))
           (set! elucidator-source-destination-delta "")
           (set! image-file-access 'fixed)
           (set! fixed-image-directory (string-append (laml-dir-prefix (html-directory)) "images/elucidator/"))
           (set! ep-stylesheet-approach 'central)
     ))

     (set! program-source-list
        (list (list (list 'key source-key) (list 'file-location source-file-path-syntactical-comments)
              (list 'language "Scheme")
              (list 'group "core") (list 'version version-number) 
              (list 'process #t)
              (list 'friendly-name "Program")
             )))
     (set! store-defined-names? #f)
     (set! manual-frame-from-program #f)
     (set! schemedoc-back-linking? #t)     

     (set! program-source-key-version-map (list (cons source-key version-number)))
     (set! make-large-source-files? #f)
     (set! cross-reference-index-support 'none)
     (set! the-scheme-report-version 'r5rs)
     (set! scheme-syntax-procedure-list (read-scheme-knowledge 5))
     (set! scheme-syntax-procedure-names (map first scheme-syntax-procedure-list))
     (set! rnrs-url-prefix 
           (string-append 
		 (laml-dir-prefix ep-dir-path)	; relative or absolute path to LAML root dir
		 "r5rs" "/"
		 ))
     (set! elucidator-verbose-mode #f)

     (set! manual-source-list
       (if laml-manual-stuff
           (append
            (map                ; the LAML lib/ manuals 
             (lambda (key) 
               (list 
                (list 'key key)
                (list 'file-location (string-append  laml-dir "lib/man/" key))  
                (list 'url-location (string-append (laml-home-url-prefix 1) "lib/man/" key ".html"))
                )
               )
             (list "cgi" "collect-skip" "color" "crypt" "encode-decode" "file-read" "final-state-automaton" "general" "time" 
                   "xhtml10-convenience")
             )
            (map               ; the LAML core library
             (lambda (key) 
               (list 
                (list 'key key)
                (list 'file-location (string-append laml-dir "man/" key))
                (list 'url-location (string-append (laml-home-url-prefix 1) "man/" key ".html"))
                )
               )
             (list "laml")
             )
            (map               ; the XML-in-LAML library
             (lambda (key) 
               (list 
                (list 'key key)
                (list 'file-location (string-append laml-dir "lib/xml-in-laml/man/" key))
                (list 'url-location (string-append (laml-home-url-prefix 1) "lib/xml-in-laml/man/" key ".html"))
                )
               )
             (list "xml-in-laml")
             )
            (map               ; the XML-in-LAML library
             (lambda (key) 
               (list 
                (list 'key key)
                (list 'file-location (string-append laml-dir "lib/xml-in-laml/mirrors/man/" key))
                (list 'url-location (string-append (laml-home-url-prefix 1) "lib/xml-in-laml/mirrors/man/" key ".html"))
                )
               )
             (list "xhtml10-transitional-mirror" "xhtml10-strict-mirror" "xhtml10-frameset-mirror")
             )

           )
           '()))

     (set! manual-name-file-map (pre-process-manual-lsp-files manual-source-list))

     ; Now elucidate source program
     (elucidate-program-source 
      source-file-path-syntactical-comments
      html-destination-file-path 
      source-list
      (make-defining-name-occurences program-source-list)        ; defined-names
      '()                               ; documented-names
      'small
      source-key 
      version-number                                 ; source-version
      "core")

     ; Copy CSS stylesheet:
     (if (not laml-manual-stuff)
         (ensure-directory-existence! (string-append manual-destination-path local-ep-dir-name "/") "stylesheets"))

     ; Copy program stylesheet:
     (if (not laml-manual-stuff)
       (let ((program-ep-software-css-filepath 
              (string-append elucidator-software-directory "stylesheets/" "program" ".css"))
             (program-target-css-filepath 
              (string-append manual-destination-path local-ep-dir-name "/" "stylesheets/" "program.css")) 
             )
         (write-text-file
          (string-append (read-text-file-if-exists program-ep-software-css-filepath))
          program-target-css-filepath)))

     ; Make images directory:
     (if (not laml-manual-stuff) (ensure-directory-existence! (string-append manual-destination-path local-ep-dir-name "/") "images"))

     ; Copy Elucidator image files:
     (if (not laml-manual-stuff)
         (copy-files 
           elucidator-image-files
           (string-append elucidator-software-directory "images/")
           (string-append manual-destination-path local-ep-dir-name "/" "images/")))
     
   ))
)

(define (extend-doc-list-with-source-references doc-list ep-dir-name ep-html-file-name)
  (cond ((null? doc-list) '())
        ((manual-page? (car doc-list))
           (cons (extend-doc-entry-with-source-reference (car doc-list) ep-dir-name ep-html-file-name) 
                 (extend-doc-list-with-source-references (cdr doc-list) ep-dir-name ep-html-file-name)))
        (else 
           (cons (car doc-list) 
                 (extend-doc-list-with-source-references (cdr doc-list) ep-dir-name ep-html-file-name)))))

(define (extend-doc-entry-with-source-reference doc-entry ep-dir-name ep-html-file-name)
  (let* ((cross-references-clause (get-defaulted-prop 'cross-references doc-entry #f))
         (title-clause (get-prop 'title doc-entry))
         (new-reference `(reference
                         "Scheme source file"
                         ,title-clause
                         ,(string-append ep-dir-name "/" ep-html-file-name "." "html" "#" title-clause))
                       ))
    (if cross-references-clause  ; is there at least one cross references clause in doc-entry?
        (append 
            (filter (negate is-cross-reference-clause?) doc-entry)
            (list `(cross-references ,@(cons new-reference (flatten (map cdr (filter is-cross-reference-clause? doc-entry))))))
        )
        (cons `(cross-references ,new-reference)
              doc-entry))))

(define (is-cross-reference-clause? doc-entry-clause)
  (eq? (car doc-entry-clause) 'cross-references))




